<?php

/**
 *	SocializeLogin Class
 *		Socialize Login and Registration Script for e107 CMS
 *		Facebook Authentication is Powered By Gigya
 *		This script was written by Lna Lore
 *	@author Lna Lore <lonalore@freemail.hu>
 *	@link http://lonalore.hu
 *	@link http://gigya.com
 *	@license http://opensource.org/licenses/gpl-license.php GNU Public License
 */

require_once("../../../class2.php");

$obj = new SocializeLogin;

class SocializeLogin {

	/**
	 *	SocializeLogin
	 *		Constructor
	 */

	function SocializeLogin() {
		// Check GET params are valid or not
		if ($this -> CheckInputArray($this -> CollectDatas())) {
			// Check email is registrated or not
			// email is registrated
			if ($this -> IsValidUserOrNot(urldecode($_GET['email']))) {
				// Check the user signature on Gigya.com
				// Check the user is logged in or not on FB
				if ($this -> VerSignature(
					str_replace(" ", "+", html_entity_decode($_GET['UID'])), 
					urldecode($_GET['email']), 
					urldecode($_GET['loginProviderUID']), 
					$this -> GetPluginConfigs())
				) {
					// Logging in with user
					$this -> DoLoginWithUser($this -> GetUserID(urldecode($_GET['email'])));
				} else {
					$this -> RedirectToBaseURL();
				}
			// emails is not registrated yet
			} else {
				if ($this -> RegistrationIsOn()) {
					// Check the user signature on Gigya.com
					// Check the user is logged in or not on FB
					if ($this -> VerSignature(
						str_replace(" ", "+", html_entity_decode($_GET['UID'])), 
						urldecode($_GET['email']), 
						urldecode($_GET['loginProviderUID']), 
						$this -> GetPluginConfigs())
					) {
						// Do a registration with email
						$uid = $this -> DoRegistration($this -> CollectDatas());
						if ((int)$uid > 0) {
							// Logging in with new user
							$this -> DoLoginWithUser($uid);
						} else {
							$this -> RedirectToBaseURL();
						}
					} else {
						$this -> RedirectToBaseURL();
					}
				} else {
					$this -> RedirectToBaseURL();
				}
			}
		} else {
			$this -> RedirectToBaseURL();
		}
	}

	/**
	 *	RegistrationIsOn
	 *		Check registration status in Plugin Configs
	 *	@return boolean
	 *		TRUE: Registration is ON
	 *		FALSE: Registration is Off
	 */

	function RegistrationIsOn() {
		$PlugConfs = $this -> GetPluginConfigs();
		if ((int)$PlugConfs['signup'] == 1) {
			return TRUE;
		} else {
			return FALSE;
		}
	}

	/**
	 *	UpdateStatusByFacebook
	 *		Update Status Message Of User On Facebook
	 *	@param array $PlugConf
	 *		Plugin Configurations
	 *	@param array $Socialize
	 *		Datas about Gigya User
	 *	@param string $Type
	 *		Type of Update Message
	 *	@param string $Message
	 *		Text to Wall
	 */

	function UpdateStatusByFacebook($PlugConf = array(), $Socialize = array(), $Type = "", $Message = "") {
		if (!is_object($tp)) { $tp = new e_parse; }
		if ($Type == "AfterRegistration") { $Message = $tp -> toText($PlugConf['UpdStsRegMsg']); }
		if ($Type == "AfterLogin") { $Message = $tp -> toText($PlugConf['UpdStsLogMsg']); }
		if (!empty($Message)) {
			$request = new GSRequest(
				$PlugConf['api_key'], 
				$PlugConf['sec_key'], 
				"socialize.setStatus"
			);
			$request -> setParam("uid", $Socialize['UID']);
			$request -> setParam("status", $Message);
			$response = $request -> send();
		}
	}

	/**
	 *	RedirectToBaseURL
	 *		Do a redirecting to the Site base URL
	 */

	function RedirectToBaseURL() {
		header('Location: ' . SITEURLBASE);
	}

	/**
	 *	VerSignature
	 *		Do a request for Gigya with params by GET
	 *		Compare Gigya datas with GET datas (must be equal)
	 *		Check user status of FB on Gigya (logged in or not)
	 *	@param string $uid
	 *		Gigya user id... long string
	 *	@param string $email
	 *		Email address from GET params
	 *	@param integer $loginProviderUIDori
	 *		Provider Facebook user ID
	 *	@param boolean $Status
	 *		It will be the return value
	 *	@return boolean $Status
	 *		TRUE: The params GET are valid and user is logged in FB
	 *		FALSE: Invalid params or user is not logged in FB
	 */

	function VerSignature($uid, $email, $loginProviderUIDori, $PlugConf = array(), $Status = TRUE) {
		include_once("GSSDK.php");
		$request = new GSRequest($PlugConf['api_key'], $PlugConf['sec_key'], "socialize.getUserInfo");
		$request -> setParam("uid", $uid);
		$response = $request -> send();
		if ($response -> getErrorCode() == 0) {
			$valid = SigUtils::validateUserSignature(
				$response -> getString("UID", ""), 
				$response -> getString("signatureTimestamp", ""), 
				$PlugConf['sec_key'], 
				$response -> getString("UIDSignature", "")
			);
			$email = $response -> getString("email", "none");
			$isLoggedIn = $response -> getBool("isLoggedIn", "0");
			$isConnected = $response -> getBool("isConnected", "0");
			$loginProviderUID = $response -> getString("loginProviderUID", "0");
			// Check datas are valid or not
			if (!$valid) { $Status = FALSE; }
			if ($email != urldecode($email)) { $Status = FALSE; }
			if ($loginProviderUIDori != urldecode($loginProviderUID)) { $Status = FALSE; }
			if (!$isLoggedIn) { $Status = FALSE; }
			if (!$isConnected) { $Status = FALSE; }
		} else {
			$Status = FALSE;
		}
		return $Status;
	}

	/**
	 *	GetPluginConfigs
	 *		Get Plugin Configs from DB
	 *	@return array $row
	 *		Contains plugin configs
	 */

	function GetPluginConfigs() {
		$sql = new db;
		$sql -> db_Select("gigya_config", "*", "id=1");
		$row = $sql -> db_Fetch();
		return $row;
	}

	/**
	 *	CollectDatas
	 *		Collect and decode GET values for actions
	 *	@param array $Datas
	 *		Empty array for return value
	 *	@return array $Datas
	 *		Array with decoded GET params
	 */

	function CollectDatas($Datas = array()) {
		$Datas['email'] = urldecode($_GET['email']);
		$Datas['firstName'] = urldecode($_GET['firstName']);
		$Datas['nickname'] = urldecode($_GET['nickname']);
		$Datas['loginProvider'] = urldecode($_GET['loginProvider']);
		$Datas['loginProviderUID'] = urldecode($_GET['loginProviderUID']);
		$Datas['UID'] = urldecode($_GET['UID']);
		return $Datas;
	}

	/**
	 *	urldecode_array
	 *		Do urldecode() on every elements of array
	 *	@param array $arr
	 *		Input array
	 *	@return array $arr
	 *		Output array
	 */

	function urldecode_array(&$arr) {
		foreach (array_keys($arr) as $key) {
			if (is_array($arr[$key])) {
				$this -> urldecode_array($arr[$key]);
			} else {
				$arr[$key] = urldecode($arr[$key]);
			}
		}
		return $arr;
	}

	/**
	 *	GetUserID
	 *		Get user ID of a registrated user by email
	 *	@param string $email
	 *		Email address of user
	 *	@return integer/boolean
	 *		integer if user is valid... return with user ID
	 *		boolean if user is not valid... FALSE
	 */

	function GetUserID($email) {
		if (!is_object($tp)) { $tp = new e_parse; }
		$sql = new db;
		if ($sql -> db_Select("user", "user_id", "user_email='" . $tp -> toDB($email) . "'")) {
			$row = $sql -> db_Fetch();
			return $row['user_id'];
		} else {
			return FALSE;
		}
	}

	/**
	 *	DoLoginWithUser
	 *		Do a login with user, and header to base URL
	 *	@param integer $uid
	 *		User ID
	 */

	function DoLoginWithUser($uid) {
		if (!USER) {
			$this -> set_cookies($uid, md5($this -> get_info('user_password', $uid)));
			$PlugConfs = $this -> GetPluginConfigs();
			$GetParams = $this -> CollectDatas();
			if ((int)$PlugConfs['UpdStsLog'] == 1 
				AND !empty($PlugConfs['UpdStsLogMsg']) 
				AND $GetParams['loginProvider'] == "facebook"
			) {
				$this -> UpdateStatusByFacebook($PlugConfs, $GetParams, "AfterLogin");
			}
			$this -> RedirectToBaseURL();
		} else {
			$this -> RedirectToBaseURL();
		}
	}

	/**
	 *	get_info
	 *		Get a piece of information about registrated user
	 *	@param string $info
	 *		Name of database table field
	 *	@param integer $id
	 *		User ID
	 *	@return string/boolean
	 *		string if user is valid... return with information
	 *		boolean if user is not valid... return with FALSE
	 */

	function get_info($info, $id) {
		$sql = new db;
		if ($sql -> db_Select("user", "*", "user_id = '" . (int)$id . "'")) {
			$row = $sql -> db_Fetch();
			return $row[$info];
		} else {
			return FALSE;
		}
	}

	/**
	 *	set_cookies
	 *		Set cookie with user datas
	 *	@param integer $id
	 *		User ID
	 *	@param string $pwd
	 *		MD5 hash of user's password
	 *	@return string
	 *		Session element with value of cookie
	 */

	function set_cookies($id, $pwd) {
		global $pref;
		$cockiename = $pref['cookie_name'] ? $pref['cookie_name'] : 'e107cookie';
		$cookieval = $id . "." . $pwd;
		if ($pref['user_tracking'] == 'session') {
			$_SESSION[$cockiename] = $cookieval;
		} else {
			cookie($cockiename, $cookieval, (time()+3600*24*30));
		}
		return $_SESSION[$cockiename];
	}

	/**
	 *	IsValidUserOrNot
	 *		Check the user is valid or not
	 *	@param string $email
	 *		Email address of user
	 *	@param boolean $Status
	 *		It will be the return value, default FALSE
	 *	@return boolean $Status
	 *		FALSE: User doesn't exist
	 *		TRUE: User exists
	 */

	function IsValidUserOrNot($email, $Status = FALSE) {
		if (!is_object($tp)) { $tp = new e_parse; }
		$sql = new db;
		$Count = $sql -> db_Select("user", "user_id", "user_email='" . $tp -> toDB($email) . "'");
		if ($Count > 0) { $Status = TRUE; }
		return $Status;
	}

	/** 
	 *	createRandomPassword
	 *		The letter l (lowercase L) and the number 1 
	 *		have been removed, as they can be mistaken 
	 *		for each other.
	 *	@return string $pass
	 *		Random password
	 */ 

	function createRandomPassword() {
		$chars = "abcdefghijkmnopqrstuvwxyz023456789";
		srand((double)microtime()*1000000);
		$i = 0;
		$pass = '';
		while ($i <= 7) {
			$num = rand()%33;
			$tmp = substr($chars, $num, 1);
			$pass = $pass . $tmp;
			$i++;
		}
		return $pass;
	}

	/**
	 *	DoRegistration
	 *		Do a registration with collected datas
	 *	@param array $Socialize
	 *		Contains datas about new user
	 *	@return integer/boolean
	 *		integer if registration success... return with ID of new user
	 *		boolean if unsuccessful... return FALSE
	 */

	function DoRegistration($Socialize) {
		if (!is_object($tp)) { $tp = new e_parse; }
		$UNArr = explode("@", $Socialize['email']);
		$nickname = $this -> username_exists($this -> getRewriteString($UNArr[0]));
		$password = $this -> createRandomPassword();
		$username = $this -> username_exists($this -> getRewriteString($UNArr[0]));
		$insert = array(
				'user_name'			=> $tp -> toDB(strip_tags($nickname)),
				'user_loginname'	=> $tp -> toDB(strip_tags($username)),
				'user_password'		=> md5($password),
				'user_email'		=> $tp -> toDB(strip_tags($Socialize['email'])),
				'user_login'		=> $tp -> toDB(strip_tags($Socialize['nickname'])),
				'user_join'			=> time()
//				'user_class'		=> '1'
			);
		$sql = new db;
		if ($sql -> db_Insert('user', $insert)) {
			$PlugConfs = $this -> GetPluginConfigs();
			$GetParams = $this -> CollectDatas();
			if ((int)$PlugConfs['UpdStsReg'] == 1 
				AND !empty($PlugConfs['UpdStsRegMsg']) 
				AND $GetParams['loginProvider'] == "facebook"
			) {
				$this -> UpdateStatusByFacebook($PlugConfs, $GetParams, "AfterRegistration");
			}
			$new_user_id = $this -> last_facebook_user($username);
			return $new_user_id;
		} else {
			return FALSE;
		}
	}

	/**
	 *	last_facebook_user
	 *		Get the User ID of registrated user
	 *	@param string $username
	 *		Username of registrated user
	 *	@return integer
	 *		User ID of registrated user
	 */

	function last_facebook_user($username) {
		if (!is_object($tp)) { $tp = new e_parse; }
		$sql = new db;
		$sql -> db_Select_gen("SELECT user_id FROM #user WHERE user_loginname = '" . $tp -> toDB($username) . "' ");
		$row = $sql -> db_Fetch();
		return $row['user_id'];
	}

	/**
	 *	username_exists
	 *		Check user name is exist or not. If it exists, append
	 *		a postfix to make it unique.
	 *	@param string $user
	 *		Username
	 *	@return string $user
	 *		Unique username
	 */

	function username_exists($user) {
		if (!is_object($tp)) { $tp = new e_parse; }
		$sql = new db;
		if ($sql -> db_Select("user", "user_loginname", "user_loginname = '" . $tp -> toDB(strip_tags($user)) . "' ")) {
			$sql2 = new db;
			$count = $sql2 -> db_Count("user", "(*)", "WHERE $name LIKE '" . $tp -> toDB(strip_tags($user)) . "%' ");
			$num = $count+1;
			return $this -> username_exists($user . $num);
		} else {
			return $user;
		}
	}

	/**
	 *	getRewriteString
	 *		Remove all illegal chars from string, and convert
	 *		special chars (for example hungarian) into English.
	 *	@param string $sString
	 *		Input string
	 *	@return string $string
	 *		Output string
	 */

	function getRewriteString($sString) {
		$string = strtolower(htmlentities($sString));
		$string = preg_replace("/&(.)(uml);/", "$1", $string);
		$string = preg_replace("/&(.)(acute|accent|cedil|circ|ring|tilde|uml|umlaut);/", "$1", $string);
		$string = preg_replace("/([^a-z0-9]+)/", "", html_entity_decode($string));
		$string = trim($string, "-");
		return $string;
	}

	/**
	 *	CheckInputArray
	 *		Check all necessary datas are available or not.
	 *	@param array $Socialize
	 *		Input array with params of GET method
	 *	@param boolean $Status
	 *		Default TRUE. It will be the return value
	 *	@return boolean $Status
	 *		Array is valid or not.
	 */

	function CheckInputArray($Socialize, $Status = TRUE) {
		if (!is_array($Socialize)) { $Status = FALSE; }
		if (empty($Socialize['email'])) { $Status = FALSE; }
		if (empty($Socialize['nickname'])) { $Status = FALSE; }
		if (empty($Socialize['loginProviderUID'])) { $Status = FALSE; }
		if (empty($Socialize['firstName'])) { $Status = FALSE; }
		if (empty($Socialize['UID'])) { $Status = FALSE; }
		return $Status;
	}

}

?>